@@ -42,7 +42,7 @@ else:
 
 class WorkerSettings(BaseModel):
 
-    debug: bool = True
+    debug: bool = False
     enabled: bool = True
 
     logging: LoggingConfig = LoggingConfig({
