@@ -4,14 +4,15 @@ import asyncio
 import logging
 from datetime import datetime
 from functools import partial
-from typing import MutableMapping, List, Union, Coroutine
+from typing import MutableMapping, List, Union, Coroutine, Type
 
 from patchwork.core import Task, AsyncSubscriber
-from patchwork.core.config.base import Dependency
+from patchwork.core.config.base import ComponentConfig
 from patchwork.core.utils import cached_property
 from patchwork.node.core import Module
 from patchwork.node.core.exceptions import TaskControlException, TaskRetry, TaskFatal, TaskDrop
-from patchwork.node.core.router import Router
+from patchwork.node.core.router import TaskRouter
+from patchwork.node.executor.unit import InlineUnit
 from patchwork.node.executor.unit.base import ProcessingUnit
 
 
@@ -25,14 +26,11 @@ class BaseExecutor(Module):
 
         # executor termination timeout in seconds
         terminate_timeout: int = 10
-        listen_on: List[str] = ('default', )
-        unit: Dependency = Dependency(
-            engine='patchwork.node.executor.unit:InlineUnit'
-        )
+        unit: ComponentConfig[Type[ProcessingUnit]] = ComponentConfig(InlineUnit)
 
     logger_name = 'executor'
     _loop_task: asyncio.Future
-    _routers: List[Router]
+    _routers: List[TaskRouter]
     _unit_monitor_task: asyncio.Future
 
     def __init__(self, *, parent, **options):
@@ -57,7 +55,7 @@ class BaseExecutor(Module):
     def unit(self) -> ProcessingUnit:
         return self.settings.unit.instantiate(parent=self)
 
-    def add_router(self, router: Router):
+    def add_router(self, router: TaskRouter):
         router.bind(self.worker)
         self._routers.append(router)
 
@@ -153,48 +151,28 @@ class BaseExecutor(Module):
         Set self._termination_request event to request the loop termination.
         :return:
         """
-        subscribers = []
-        for c_name in self.settings.listen_on:
-            try:
-                s = self.worker.get_subscriber(c_name)
-                s.name = c_name
-                subscribers.append(s)
-            except KeyError as e:
-                self.logger.error(f"Can't listen on client {e}. No such client.")
-
         # future of termination request
-        term_future = asyncio.ensure_future(self._terminate_request.wait())
-        msg_waiters = {asyncio.ensure_future(c.get()): c for c in subscribers}
-
-        self.logger.debug(f"Listening on {len(msg_waiters)} subscribers: {', '.join(c.name for c in subscribers)}")
+        term_future = asyncio.create_task(self._terminate_request.wait())
+        subscriber = self.worker.get_subscriber()
 
         while True:
-            if not msg_waiters:
-                self.logger.error(f"No subscribers to listen on, executor can't continue")
-                raise RuntimeError("No subscribers to listen on")
 
             try:
-                done, pending = await asyncio.wait(tuple(msg_waiters.keys()) + (term_future,),
+                done, pending = await asyncio.wait((asyncio.create_task(subscriber.get()), term_future,),
                                                    return_when=asyncio.FIRST_COMPLETED)
             except Exception as e:
                 self.logger.error(f"Client get() raises exception: {e.__class__.__name__}({e})",
                                   exc_info=True)
             else:
-                if self._terminate_request.is_set():
-                    # termination future done, even if there is more done futures skip them
-                    # worker is going down
-                    for fut in pending:
-                        fut.cancel()
-                    return
+                for task in done:
+                    if task == term_future:
+                        # termination future done, even if there is more done futures skip them
+                        # worker is going down
+                        for fut in pending:
+                            fut.cancel()
+                        return
 
-                for fut in done:
-
-                    # remove completed future from the waiters map
-                    client = msg_waiters.pop(fut)
-
-                    if await self._handle_done_waiter(fut, client):
-                        # re-append waiter for the client
-                        msg_waiters[asyncio.ensure_future(client.get())] = client
+                    await self._handle_done_waiter(task, subscriber)
 
     async def _handle_done_waiter(self, future: asyncio.Future, receiver: AsyncSubscriber) -> bool:
         """
@@ -315,7 +293,7 @@ class BaseExecutor(Module):
                     return task_future
 
             self.logger.error(f"No processor to handle message type '{task.task_type}'")
-            raise TaskFatal(task, reason="No processor to handle this message type") from e
+            raise TaskFatal(task, reason="No processor to handle this message type")
 
         except TaskControlException as exc:
             # allow _execute to raise task control exceptions
